#include "igc.h"

Fix::Fix(const QString &s)
{
    int pos = 0;

    time_ = QTime::fromString(s.mid(pos, 6), "hhmmss"); 
    pos += 6;	

    gps_.lat = s.mid(pos, 8); pos += 8;	// DDMMmmm[NS]
    gps_.lon = s.mid(pos, 9); pos += 9;	// DDDMMmmm[EW]

    is3d_ = s.mid(pos, 1) == "A"; 
    pos += 1;

    alt_[baro] = s.mid(pos, 5).toInt(); pos += 5;
    alt_[gps]  = s.mid(pos, 5).toInt(); pos += 5;
}


IGC::IGC()
#ifdef __GXX_EXPERIMENTAL_CXX0X__
    : altitude_({0})
#endif
{
#ifndef __GXX_EXPERIMENTAL_CXX0X__
    memset(altitude_, 0, sizeof(altitude_));
#endif
}

IGC::IGC(QTextStream &s)
#ifdef __GXX_EXPERIMENTAL_CXX0X__
    : altitude_({0})
#endif
{
#ifndef __GXX_EXPERIMENTAL_CXX0X__
    memset(altitude_, 0, sizeof(altitude_));
#endif
    read(s);
}

IGC::~IGC()
{
    for (size_t i = 0; i < sizeof(altitude_)/sizeof(altitude_[0]); ++i) {
	delete altitude_[i];
    }
}

void IGC::read(QTextStream &in)
{
    while (!in.atEnd()) {
	char c;
	in >> c;
	switch (c) {
	    case 'A': 
		device_ = in.readLine(); 
		break;
	    case 'H': {
		QString line  = in.readLine();
		QString key   = line.section(':', 0, 0);
		QString value = line.section(':', 1, 1).simplified();

        if (key.startsWith("FPLT")) pilot_ = value;
		else if (key.startsWith("FGTY")) glider_ = value;
		else if (key.startsWith("FDTE")) {
		    QString s = line.mid(4);
		    if (s.length() == 6) 
			s.insert(4, "20"); // FIXME: current year?
		    date_ = QDate::fromString(s, "ddMMyyyy");
		    }
		}
		break;
	    case 'B':  {
		Fix fix(in.readLine());
		if (!fixes_.count()) start_ = fix;
		landing_ = fix;
		fixes_ << fix;
		}
		break;
	    default:
		(void) in.readLine();
	}
    }
}

const QVector<QPointF>& IGC::altitude(Fix::Source src) const
{
   
    // optimization: if already allocated, return the allocated vector
    if (altitude_[src]) return *altitude_[src];

    altitude_[src] = new QVector<QPointF>;

    for (Fixes::const_iterator i = fixes_.begin();
	     i != fixes_.end();
	     i++) {
	     *altitude_[src] << QPointF(QTime().secsTo(i->time()), i->altitude(src));
    }

    return *altitude_[src];
}
