#include "mainwindow.h"
#include <qwt_plot_curve.h>

#include "ui_mainwindow.h"
#include "plot.h"

#include "release.h"


using Qt::DefaultLocaleLongDate;

MainWindow::MainWindow(QWidget *parent)
    : QMainWindow(parent), ui_(new Ui::MainWindow),
    igc_(0),
    plot_(new QwtPlot(tr("Altitude"), this))
{
    ui_->setupUi(this);
    ui_->tabs->setCurrentIndex(0);	// avoid changes done in the designer!

    // prepare the Qwt tab
    plot_->setAxisTitle(plot_->yLeft, tr("Altitude (m)", "axis"));
    plot_->setAxisTitle(plot_->xBottom, tr("Time"));

    ui_->qwt->setLayout(new QGridLayout);
    ui_->qwt->layout()->addWidget(plot_);

#ifndef MYPLOT
    ui_->tabs->removeTab(1);		// altitude
#endif

    connect(ui_->actionAbout, SIGNAL(triggered()), this, SLOT(about()));
    connect(ui_->actionAboutQt, SIGNAL(triggered()), qApp, SLOT(aboutQt()));
    connect(ui_->actionExit, SIGNAL(triggered()), qApp, SLOT(quit()));
    connect(ui_->actionOpen, SIGNAL(triggered()), this, SLOT(open()));


    /*
    QNetworkProxy::setApplicationProxy(QNetworkProxy(QNetworkProxy::HttpProxy, "localhost", 3128));
    ui_->view->load(QUrl("http://maps.google.com/"));
    */


}

MainWindow::~MainWindow()
{
    delete ui_;
    delete igc_;
    delete plot_;
}

void MainWindow::about() const
{
    QMessageBox::information(0, tr("About"), tr("IGC-Viewer %1").arg(RELEASE));
}

void MainWindow::open(const QString &s)
{
    QString fileName;
    if (s.isEmpty()) {
	fileName = QFileDialog::getOpenFileName(
		this, 
		tr("Open IGC file"), 
		"flights", tr("IGC files (*.igc);; All files (*)"));
	if (fileName.isEmpty()) return;
    }
    else fileName = s;


    if(currentFile_.isOpen()) 
	currentFile_.close();

    currentFile_.setFileName(fileName);

    if (!currentFile_.open(QFile::ReadOnly)) {
	QMessageBox::critical(this, 
	    tr("Ooops!"),
	    tr("The file %1 cannot be opened: %2!").arg(fileName).arg(currentFile_.errorString()));
	return;
    }

    input_.setDevice(&currentFile_);
    setWindowTitle(QFileInfo(fileName).baseName());
    statusBar()->showMessage(tr("opening %1").arg(fileName), 5000);

    delete igc_;
    igc_ = new IGC(input_);

    showInfo();
    showProfile();

}

void MainWindow::showInfo()
{
    // now let's fill the table
    QTableWidget *t = ui_->table;
    int r = 0;
    const int K = 0;
    const int V = 1;

    // cleanup
    t->clearContents();
    t->setRowCount(0);

    // fill
    t->setRowCount(t->rowCount() + 1);
    t->setItem(r, K, new QTableWidgetItem(tr("Pilot")));
    t->setItem(r, V, new QTableWidgetItem(igc_->pilot()));
    ++r;

    t->setRowCount(t->rowCount() + 1);
    t->setItem(r, K, new QTableWidgetItem(tr("Date")));
    t->setItem(r, V, new QTableWidgetItem(igc_->date()
	.toString(DefaultLocaleLongDate)));
    ++r;

    t->setRowCount(t->rowCount() + 1);
    t->setItem(r, K, new QTableWidgetItem(tr("Glider")));
    t->setItem(r, V, new QTableWidgetItem(igc_->glider()));
    ++r;

    t->setRowCount(t->rowCount() + 1);
    t->setItem(r, K, new QTableWidgetItem(tr("Place")));
    t->setItem(r, V, new QTableWidgetItem(igc_->start().longitude() + " / " + igc_->start().latitude()));
    ++r;


}

void MainWindow::showProfile()
{
	QVector<QPointF> pointsGPS = igc_->altitude(Fix::gps);
	QVector<QPointF> pointsBaro = igc_->altitude(Fix::baro);

	QwtPlotCurve *gps = new QwtPlotCurve(tr("Altitude GPS"));
	QwtPlotCurve *baro = new QwtPlotCurve(tr("Altitude barometric"));
	QwtPointSeriesData *dataGPS = new QwtPointSeriesData(pointsGPS);
	QwtPointSeriesData *dataBaro = new QwtPointSeriesData(pointsBaro);
	
	gps->setData(dataGPS);
	baro->setData(dataBaro);
	gps->attach(plot_);
	baro->attach(plot_);

   // ui_->altitude->show();
}
